// ---------------------------------------------------------------------
// Именованные константы
// ---------------------------------------------------------------------

localparam
    fetch           = 0,
    fetch_modrm     = 1,
    exec            = 2,
    modrm_wb        = 3,
    fetch_imm16     = 4;

localparam
    CF = 0, PF = 2, AF =  4, ZF =  6, SF = 7,
    TF = 8, IF = 9, DF = 10, OF = 11,
    IOPL0 = 12,
    IOPL1 = 13,
    NT    = 14, RF    = 16, VM = 17;

localparam
    alu_add = 3'h0, alu_or  = 3'h1,
    alu_adc = 3'h2, alu_sbb = 3'h3,
    alu_and = 3'h4, alu_sub = 3'h5,
    alu_xor = 3'h6, alu_cmp = 3'h7;

// ---------------------------------------------------------------------
// УПРАВЛЕНИЕ ПАМЯТЬЮ
// ---------------------------------------------------------------------

// Пока что режим REAL-MODE
assign address = src ? {segment[15:0], 4'h0} +  ea[15:0]: // Если segment:ea
                       {     cs[15:0], 4'h0} + eip[15:0]; // Иначе cs:eip

initial begin we = 1'b0; out = 1'b0; end

// ---------------------------------------------------------------------
// РЕГИСТРЫ И СЕГМЕНТЫ
// ---------------------------------------------------------------------

// 8 x 32 битных регистров общего назначения
reg [31:0]  eax = 32'h0000_1234;
reg [31:0]  ebx = 32'h0000_AABC;
reg [31:0]  ecx = 32'h0000_0000;
reg [31:0]  edx = 32'h0000_0000;
reg [31:0]  esp = 32'h0000_0000;
reg [31:0]  ebp = 32'h0000_0000;
reg [31:0]  esi = 32'h0000_0000;
reg [31:0]  edi = 32'h0000_0000;

//                        VR  Nio ODIT SZ A P1C
reg [17:0]  eflags  = 18'b00_0000_0000_00000010;
reg [31:0]  eip     = 32'h0000_0000;

//                   | ДЕСКРИПТОР        | СЕЛЕКТОР
reg [79:0]  es  = 80'h0000_0000_0000_0000_____0000;
reg [79:0]  cs  = 80'h0000_0000_0000_0000_____F000;
reg [79:0]  ss  = 80'h0000_0000_0000_0000_____0000;
reg [79:0]  ds  = 80'h0000_0000_0000_0000_____0000;
reg [79:0]  fs  = 80'h0000_0000_0000_0000_____0000;
reg [79:0]  gs  = 80'h0000_0000_0000_0000_____0000;
// ---------------------------------------------------------------------

reg [3:0]   t               = 1'b0;     // Фаза исполнения
reg [3:0]   t_next          = 1'b0;     // Фаза исполнения
reg [4:0]   fn              = 1'b0;     // Фаза exec
reg [3:0]   fn2             = 1'b0;     // Фаза процедур
reg [7:0]   opcode          = 1'b0;     // Сохраненный опкод
reg [7:0]   modrm           = 1'b0;     // Сохраненный modrm
reg [2:0]   regn            = 1'b0;     // И SIB-байт
reg         src             = 1'b0;     // Источник адреса segment:ea; cs:eip
reg [79:0]  segment         = 1'b0;     // Рабочий сегмент
reg [31:0]  ea              = 1'b0;     // Эффективный адрес
reg         prot            = 1'b0;     // =1 Защищенный режим
reg         adsize          = 1'b0;     // =1 32х битная адресация
reg         opsize          = 1'b0;     // =1 32х битный операнд
reg         override        = 1'b0;     // =1 Сегмент префиксирован
reg         ignoreo         = 1'b0;     // Игнорировать чтение из памяти modrm
reg [1:0]   rep             = 1'b0;     // Режим REPNZ/REPZ
reg [2:0]   alu             = 3'h0;     // Режим АЛУ
reg         size            = 1'b0;     // =1 16/32 битный операнд
reg         dir             = 1'b0;     // =0 rm,r; =1 r,rm modrm
reg [31:0]  op1             = 32'h0;    // Левый операнд
reg [31:0]  op2             = 32'h0;    // Правый операнд
reg [31:0]  wb              = 32'h0;    // Значение для записи

reg         __adsize        = 1'b0;
reg         __opsize        = 1'b0;
reg         __override      = 1'b0;
reg [1:0]   __rep           = 2'b00;
reg [79:0]  __segment       = 16'h0000;

// ---------------------------------------------------------------------
// Вычисление следующего EIP в зависимости от 54-го бита
// ---------------------------------------------------------------------

wire        defsize     = cs[54+16] & prot;
wire [15:0] ipnext1     = eip[15:0] + 1'b1;
wire [15:0] ipnext2     = eip[15:0] + 2'h2;
wire [31:0] eip_next    = defsize ? eip + 1'b1 : {eip[31:16], ipnext1};
wire [31:0] eip_next2   = defsize ? eip + 2'h2 : {eip[31:16], ipnext2};

wire [31:0] sib_base =
    in[2:0] == 3'b000 ? eax :
    in[2:0] == 3'b001 ? ecx :
    in[2:0] == 3'b010 ? edx :
    in[2:0] == 3'b011 ? ebx :
    in[2:0] == 3'b100 ? esp :
    in[2:0] == 3'b101 ? (^modrm[7:6] ? ebp : 1'b0) :
    in[2:0] == 3'b110 ? esi : edi;
